/*
   So far we have a "tree language" for describing trees,
   and a "parser" that can convert strings from that language
   into Java tree data structures. We also have a "arithmetic
   language" that is a subset of the tree language.

   If we take an arithmetic expression like
       3 * 4 + 5 + 6
   and write it in its "prefix" notation
      ( + ( * 3 4 ) 5 6 )
   we now have a string in our arithmetic language (and
   also a string  in ourtree language), so we can use our
   parser to build the "abstract syntax tree" for this
   expression. Once we have the AST for the expression,
   we can evaluate it using a simple post-order traversal
   of the AST.
*/

public class Language_1_Example_2
{
   public static void main(String[] args) throws ParseException, TokenizeException, EvalException
   {
      String[] treeStrings = {
                               "(+ (* 3 4) 5 6)",  // 3*4+5+6
                               "(* 3 (+ 4 5 6))",  // 3*(4+5+6)
                               "(+ 3 4 5 6)",      // 3+4+5+6
                               "(+ 3 4 (* 5 6))"   // 3+4+5*6
                             };

      int i = 0;
      for (i = 0; i < treeStrings.length; i++)
      {
         System.out.println(i + " =============================================");

         // Build the abstract syntax tree that represents the expression.
         Tree ast = ParseTree.buildTree( treeStrings[i] );

         // Print the AST as an S-expression (using Tree's toString() method).
         System.out.println( "Original expression is " + ast + "\n" );

         // Pretty-print the abstract syntax tree (expression tree).
         System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

         // Print the infix version of the expression.
         System.out.println("Infix expression = " + AST2infix.ast2infix(  ast ) + "\n");
         System.out.println("Infix expression = " + AST2infix2.ast2infix( ast ) + "\n");

         // Evaluate the expression (interpret the AST).
         int result = Evaluate.eval( ast );
         System.out.println( ast + " ==> " + result + "\n" );


         // Create dot and png files from the ast.
         if(true)
         try
         {
            // Create the (empty) dot file.
            String baseName = String.format("Example_2.%02d", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                         new java.io.File(baseName + ".dot"));
            // Put dot commands into the dot file
            out.println( Tree2dot.tree2dot(ast) + "\n" );
            out.close();

            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()
}
